
type 'a t = {
  buckets: 'a list array;
  mutable size: int;
}

let create n =
  { buckets = Array.make n []; size = 0 }

let size t =
  t.size

let rec lookup p l = match l with
  | []     -> None
  | x :: l -> if p x then Some x else lookup p l

(* Il est important de se prémunir contre une fonction hash qui renverrait
   une valeur négative (ce qui arrive facilement par débordement arithmétique).
   Utiliser abs pour cela serait incorrect, car abs min_int = min_int.
   On choisit ici de masquer le bit de signe, mais d'autres solutions sont
   possibles. *)
let index hash x n =
  (hash x land max_int) mod n

let find hash eq t x =
  let n = Array.length t.buckets in
  let i = index hash x n in
  lookup (eq x) t.buckets.(i)

let add hash eq t x =
  let n = Array.length t.buckets in
  let i = index hash x n in
  let b = t.buckets.(i) in
  if lookup (eq x) b = None then begin
    t.buckets.(i) <- x :: b;
    t.size <- t.size + 1
  end

(* non demandée *)
let find_or_add hash eq t x =
  let n = Array.length t.buckets in
  let i = index hash x n in
  let b = t.buckets.(i) in
  match lookup (eq x) b with
    | None ->
      t.buckets.(i) <- x :: b;
      t.size <- t.size + 1;
      x
    | Some y ->
      y
