
(* Une variante, à la Caml Light : les fonctions [hash] et [eq] sont
   passées lors de la création et stockées dans la structure.
   C'est plus sûr, car on maintient maintenant la cohérence de [hash]
   et [eq] entre différentes opérations sur une même table. *)

type 'a t = {
  hash: 'a -> int;
  eq: 'a -> 'a -> bool;
  buckets: 'a list array;
  mutable size: int;
}

let create hash eq n =
  { hash = hash; eq = eq; buckets = Array.make n []; size = 0 }

let size t =
  t.size

let rec lookup p l = match l with
  | [] -> None
  | x :: l -> if p x then Some x else lookup p l

let index t x =
  (t.hash x land max_int) mod (Array.length t.buckets)

let add t x =
  let i = index t x in
  let b = t.buckets.(i) in
  if lookup (t.eq x) b = None then begin
    t.buckets.(i) <- x :: b;
    t.size <- t.size + 1
  end

let find t x =
  let i = index t x in
  lookup (t.eq x) t.buckets.(i)

