
open Tree

let rec perfect n = match n with
  | 0 -> E
  | n -> N (perfect (n-1), n, perfect (n-1))

(* les arbres restent les mêmes *)
let () =
  Format.printf "Je teste share avec des arbres binaires parfaits.@.";
  assert (share (perfect 0) = perfect 0);
  assert (share (perfect 1) = perfect 1);
  assert (share (perfect 2) = perfect 2);
  assert (share (perfect 3) = perfect 3);
  assert (share (perfect 4) = perfect 4);
  Format.printf "OK@."

(* et ils ont la bonne taille *)
let () =
  Format.printf "Et je vérifie aussi le résultat de mem_size.@.";
  assert (mem_size (share (perfect 0)) = 0);
  assert (mem_size (share (perfect 1)) = 1);
  assert (mem_size (share (perfect 2)) = 2);
  assert (mem_size (share (perfect 3)) = 3);
  assert (mem_size (share (perfect 4)) = 4);
  Format.printf "OK@."

(* d'autres tests, avec des arbres pas forcément symétriques *)

let rec level ofs n = match n with
  | 0 -> E
  | n -> N (level ofs (n-1), ofs + n mod 2, level (1 - ofs) (n-1))

let () =
  Format.printf "Je teste share avec d'autres arbres binaires.@.";
 assert (share (level 0 0) = level 0 0);
  assert (share (level 0 1) = level 0 1);
  assert (share (level 0 2) = level 0 2);
  assert (share (level 0 3) = level 0 3);
  assert (share (level 0 4) = level 0 4);
  Format.printf "OK@."

let () =
  Format.printf "Et je vérifie aussi le résultat de mem_size.@.";
  assert (mem_size (share (level 0 0)) = 0);
  assert (mem_size (share (level 0 1)) = 1);
  assert (mem_size (share (level 0 2)) = 3);
  assert (mem_size (share (level 0 3)) = 5);
  assert (mem_size (share (level 0 4)) = 7);
  Format.printf "OK@."

let rec fib n = match n with
  | 0 | 1 as n -> N (E, n, E)
  | n -> N (fib (n-2), n, fib (n-1))

let () =
  Format.printf "Je teste share avec des arbres de Fibonacci.@.";
  assert (mem_size (share (fib 0)) = 1);
  assert (mem_size (share (fib 1)) = 1);
  assert (mem_size (share (fib 2)) = 3);
  assert (mem_size (share (fib 3)) = 4);
  assert (mem_size (share (fib 4)) = 5);
  Format.printf "OK@.";
  Format.printf "Tous les tests sont passés avec succès.@."


